package com.rjconsultores.ventaboletos.web.gui.controladores.impressaofiscal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FiscalFormapagoEmpresa;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;

@Controller("busquedaFormapagoEmpresaController")
@Scope("prototype")
public class BusquedaFormapagoEmpresaController extends MyGenericForwardComposer  {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	@Autowired
    private transient PagedListWrapper<FiscalFormapagoEmpresa> plwFormapagoEmpresa;
	private Paging pagingFormapagoEmpresa;
	
	private Combobox cmbEmpresa;
	private List<Empresa> lsEmpresas;
	
	private MyListbox formapagoEmpresaList;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {		
		
		lsEmpresas = UsuarioLogado.getUsuarioLogado().getEmpresa();
		
		super.doAfterCompose(comp);
		
		formapagoEmpresaList.setItemRenderer(new FormapagoEmpresaListItemRenderer());
		formapagoEmpresaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				FiscalFormapagoEmpresa formapagoEmpresa = (FiscalFormapagoEmpresa) formapagoEmpresaList.getSelected();
				visualizaFormapagoEmpresa(formapagoEmpresa);
			}
		});
		
		actualizaFormapagoEmpresaList();
	}
	
	private void visualizaFormapagoEmpresa(FiscalFormapagoEmpresa formapagoEmpresa){
		
		if (formapagoEmpresa == null)
			return;
		
		Map args = new HashMap();
		args.put("formapagoEmpresa", formapagoEmpresa);
		args.put("formapagoEmpresaList", formapagoEmpresaList);

		openWindow("/gui/impressaofiscal/editarFormapagoEmpresa.zul",
				Labels.getLabel("editarFormapagoEmpresaController.window.title"), args, MODAL);
		
	}
	
	public void onClick$btnPesquisa(Event ev) throws InterruptedException {
		actualizaFormapagoEmpresaList();
	}

	public void onClick$btnRefresh(Event ev) {
		actualizaFormapagoEmpresaList();
	}

	public void onClick$btnNovo(Event ev) {
		visualizaFormapagoEmpresa(new FiscalFormapagoEmpresa());
	}
	
	private void actualizaFormapagoEmpresaList() {
		HibernateSearchObject<FiscalFormapagoEmpresa> editarFormapagoEmpresaBusqueda =
				new HibernateSearchObject<FiscalFormapagoEmpresa>(FiscalFormapagoEmpresa.class, pagingFormapagoEmpresa.getPageSize());

		editarFormapagoEmpresaBusqueda.addFilterEqual(ACTIVO, true);
		
		Comboitem itemEmpresa = cmbEmpresa.getSelectedItem();		
		if (itemEmpresa != null) {
			Empresa empresa = (Empresa) itemEmpresa.getValue();
			editarFormapagoEmpresaBusqueda.addFilterEqual("empresa", empresa);
		} else {
			editarFormapagoEmpresaBusqueda.addFilterIn("empresa", lsEmpresas);
		}
		
		plwFormapagoEmpresa.init(editarFormapagoEmpresaBusqueda, formapagoEmpresaList, pagingFormapagoEmpresa);

		if (formapagoEmpresaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaAidfController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}
	
	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> _lsEmpresas) {
		lsEmpresas = _lsEmpresas;
	}
	
}
